from ctypes import cdll
import ctypes

try:
    lib = cdll.LoadLibrary('/usr/lib/libastraevents.so')
    if lib.asev_init() < 0:
        print("Can not initialize libastraevents!")
        exit(-1)
except Exception as e:
    print(e)
    exit(-1)

DEFAULT_FACILITY  = -1
DEFAULT_LEVEL     = -1

NO_FLAGS          = 0
VERBOSE           = 0x01
GET_DESCRIPTIONS  = 0x02

lib.asev_get_event_ids.restype = ctypes.POINTER(ctypes.c_char_p * lib.asev_events_count())
lib.asev_get_group_ids.restype = ctypes.POINTER(ctypes.c_char_p * lib.asev_groups_count())
lib.asev_get_event_ids_ex.restype = ctypes.POINTER(ctypes.c_char_p * lib.asev_events_count())
lib.asev_get_group_ids_ex.restype = ctypes.POINTER(ctypes.c_char_p * lib.asev_groups_count())

def events_count() -> int:
    return lib.asev_events_count()

def groups_count() -> int:
    return lib.asev_groups_count()

def get_event_ids() -> list:
    result = []
    ptr = lib.asev_get_event_ids()
    for event in ptr.contents:
        result.append(event.decode("utf-8"))
    return result

def get_event_ids_ex(flags: int) -> list:
    result = []
    ptr = lib.asev_get_event_ids_ex(flags)
    for event in ptr.contents:
        result.append(event.decode("utf-8"))
    return result

def get_group_ids() -> list:
    result = []
    ptr = lib.asev_get_group_ids()
    for event in ptr.contents:
        result.append(event.decode("utf-8"))
    return result

def get_group_ids_ex(flags: int) -> list:
    result = []
    ptr = lib.asev_get_group_ids_ex(flags)
    for event in ptr.contents:
        result.append(event.decode("utf-8"))
    return result

def is_event_enabled(event_id: str) -> int:
    return lib.asev_is_event_enabled(bytes(event_id, "utf-8"))

def enable_event(event_id: str) -> int:
    return lib.asev_enable_event(bytes(event_id, "utf-8"))

def disable_event(event_id: str) -> int:
    return lib.asev_disable_event(bytes(event_id, "utf-8"))

def get_event_param(event_id: str, section: str, parameter: str) -> str:
    result = lib.asev_get_event_param(bytes(event_id, "utf-8"),
                                      bytes(section, "utf-8") if section else None,
                                      bytes(parameter, "utf-8"))
    return ctypes.c_char_p(result).value.decode("utf-8")

def set_event_param(event_id: str, section: str,
                    parameter: str, value: str) -> int:
    return lib.asev_set_event_param(bytes(event_id, "utf-8"),
                                    bytes(section, "utf-8") if section else None,
                                    bytes(parameter, "utf-8"),
                                    bytes(value, "utf-8"))

def add_event_to_notifyrc(event_id: str, notifyrc_file: str) -> int:
    return lib.asev_add_event_to_notifyrc(bytes(event_id, "utf-8"), bytes(notifyrc_file, "utf-8"))

def restart_daemons_if_need() -> int:
    return lib.asev_restart_daemons_if_need()

def free() -> None:
    return lib.asev_free()

class ExtendedEventParams:
    def __init__(self):
        self.extended_params = lib.asev_new_extended_event_params()

    def set_param(self, param_name: str, param_value: str) -> None:
        return lib.asev_set_extended_param(self.extended_params, bytes(param_name, "utf-8"), bytes(param_value, "utf-8"))

    def set_uid(self, uid: int) -> None:
        return lib.asev_set_extended_param_uid(self.extended_params, uid)

    def set_user(self, user: str) -> None:
        return lib.asev_set_extended_param_user(self.extended_params, bytes(user, "utf-8"))

    def set_gid(self, gid: int) -> None:
        return lib.asev_set_extended_param_gid(self.extended_params, gid)

    def set_group(self, group: str) -> None:
        return lib.asev_set_extended_param_group(self.extended_params, bytes(group, "utf-8"))

    def set_subj(self, subj: str) -> None:
        return lib.asev_set_extended_param_subj(self.extended_params, bytes(subj, "utf-8"))

    def set_obj(self, obj: str) -> None:
        return lib.asev_set_extended_param_obj(self.extended_params, bytes(obj, "utf-8"))

    def set_exe(self, exe: str) -> None:
        return lib.asev_set_extended_param_exe(self.extended_params, bytes(exe, "utf-8"))

    def set_cmdargs(self, cmdargs: str) -> None:
        return lib.asev_set_extended_param_cmdargs(self.extended_params, bytes(cmdargs, "utf-8"))

    def set_path(self, path: str) -> None:
        return lib.asev_set_extended_param_path(self.extended_params, bytes(path, "utf-8"))

    def set_file(self, file: str) -> None:
        return lib.asev_set_extended_param_file(self.extended_params, bytes(file, "utf-8"))

    def set_desc(self, desc: str) -> None:
        return lib.asev_set_extended_param_desc(self.extended_params, bytes(desc, "utf-8"))

    def set_success(self, success: int) -> None:
        return lib.asev_set_extended_param_success(self.extended_params, success)

    def set_unixtime(self, unixtime: int) -> None:
        return lib.asev_set_extended_param_unixtime(self.extended_params, unixtime)

    def set_time(self, time: str) -> None:
        return lib.asev_set_extended_param_time(self.extended_params, bytes(time, "utf-8"))

    def free(self):
        return lib.asev_free_extended_params(self.extended_params)

    def get(self):
        return self.extended_params

class CustomEvent:
    def __init__(self, event_id: str, name_en: str = "", name_ru: str = "",
                 facility: int = DEFAULT_FACILITY, level: int = DEFAULT_LEVEL):
        self.required_params = lib.asev_new_required_event_params2(bytes(event_id, "utf-8"),
                                                        bytes(name_en, "utf-8"),
                                                        bytes(name_ru, "utf-8"),
                                                        facility, level)

    def create(self) -> int:
        return lib.asev_create_event(self.required_params)

    def create_with_extended(self, extended_params: ExtendedEventParams) -> int:
        return lib.asev_create_event_ex(self.required_params, extended_params.get())

    def set_name_en(self, name_en: str) -> None:
        return lib.asev_set_required_param_name_en(self.required_params, bytes(name_en, "utf-8"))

    def set_name_ru(self, name_ru: str) -> None:
        return lib.asev_set_required_param_name_en(self.required_params, bytes(name_ru, "utf-8"))

    def set_facility(self, facility: int) -> None:
        return lib.asev_set_required_param_facility(self.required_params, facility)

    def set_level(self, level: int) -> None:
        return lib.asev_set_required_param_level(self.required_params, level)

    def register(self) -> int:
        return lib.asev_register_event(self.required_params)

    def register_with_extended(self, extended_params: ExtendedEventParams) -> int:
        return lib.asev_register_event_ex(self.required_params, extended_params.get())

    def create_and_register(self) -> int:
        return lib.asev_create_and_register_event(self.required_params)

    def create_and_register_with_extended(self, extended_params: ExtendedEventParams) -> int:
        return lib.asev_create_and_register_event_ex(self.required_params, extended_params.get())
